@@ -1,4 +1,6 @@
 #include "Python.h"
+#include "internal/import.h"
+#include "internal/pystate.h"
 #include "structmember.h"
 #include "osdefs.h"
 #include "marshal.h"
@@ -54,28 +56,54 @@ static PyObject *get_data(PyObject *archive, PyObject *toc_entry);
 static PyObject *get_module_code(ZipImporter *self, PyObject *fullname,
                                  int *p_ispackage, PyObject **p_modpath);
 
+static PyTypeObject ZipImporter_Type;
 
 #define ZipImporter_Check(op) PyObject_TypeCheck(op, &ZipImporter_Type)
 
+/*[clinic input]
+module zipimport
+class zipimport.zipimporter "ZipImporter *" "&ZipImporter_Type"
+[clinic start generated code]*/
+/*[clinic end generated code: output=da39a3ee5e6b4b0d input=9db8b61557d911e7]*/
+#include "clinic/zipimport.c.h"
+
 
 /* zipimporter.__init__
    Split the "subdirectory" from the Zip archive path, lookup a matching
    entry in sys.path_importer_cache, fetch the file directory from there
    if found, or else read it from the archive. */
+
+/*[clinic input]
+zipimport.zipimporter.__init__
+
+    archivepath as path: object(converter="PyUnicode_FSDecoder")
+        A path-like object to a zipfile, or to a specific path inside
+        a zipfile.
+    /
+
+Create a new zipimporter instance.
+
+'archivepath' must be a path-like object to a zipfile, or to a specific path
+inside a zipfile. For example, it can be '/tmp/myimport.zip', or
+'/tmp/myimport.zip/mydirectory', if mydirectory is a valid directory inside
+the archive.
+
+'ZipImportError' is raised if 'archivepath' doesn't point to a valid Zip
+archive.
+
+The 'archive' attribute of the zipimporter object contains the name of the
+zipfile targeted.
+
+[clinic start generated code]*/
+
 static int
-zipimporter_init(ZipImporter *self, PyObject *args, PyObject *kwds)
+zipimport_zipimporter___init___impl(ZipImporter *self, PyObject *path)
+/*[clinic end generated code: output=141558fefdb46dc8 input=92b9ebeed1f6a704]*/
 {
-    PyObject *path, *files, *tmp;
+    PyObject *files, *tmp;
     PyObject *filename = NULL;
     Py_ssize_t len, flen;
 
-    if (!_PyArg_NoKeywords("zipimporter()", kwds))
-        return -1;
-
-    if (!PyArg_ParseTuple(args, "O&:zipimporter",
-                          PyUnicode_FSDecoder, &path))
-        return -1;
-
     if (PyUnicode_READY(path) == -1)
         return -1;
 
@@ -137,10 +165,10 @@ zipimporter_init(ZipImporter *self, PyObject *args, PyObject *kwds)
     }
     else
         Py_INCREF(files);
-    self->files = files;
+    Py_XSETREF(self->files, files);
 
     /* Transfer reference */
-    self->archive = filename;
+    Py_XSETREF(self->archive, filename);
     filename = NULL;
 
     /* Check if there is a prefix directory following the filename. */
@@ -149,7 +177,7 @@ zipimporter_init(ZipImporter *self, PyObject *args, PyObject *kwds)
                                   PyUnicode_GET_LENGTH(path));
         if (tmp == NULL)
             goto error;
-        self->prefix = tmp;
+        Py_XSETREF(self->prefix, tmp);
         if (PyUnicode_READ_CHAR(path, len-1) != SEP) {
             /* add trailing SEP */
             tmp = PyUnicode_FromFormat("%U%c", self->prefix, SEP);
@@ -158,8 +186,9 @@ zipimporter_init(ZipImporter *self, PyObject *args, PyObject *kwds)
             Py_SETREF(self->prefix, tmp);
         }
     }
-    else
-        self->prefix = PyUnicode_New(0, 0);
+    else {
+        Py_XSETREF(self->prefix, PyUnicode_New(0, 0));
+    }
     Py_DECREF(path);
     return 0;
 
@@ -294,6 +323,12 @@ get_module_info(ZipImporter *self, PyObject *fullname)
     PyObject *path, *fullpath, *item;
     struct st_zip_searchorder *zso;
 
+    if (self->prefix == NULL) {
+        PyErr_SetString(PyExc_ValueError,
+                        "zipimporter.__init__() wasn't called");
+        return MI_ERROR;
+    }
+
     subname = get_subname(fullname);
     if (subname == NULL)
         return MI_ERROR;
@@ -379,21 +414,30 @@ find_loader(ZipImporter *self, PyObject *fullname, PyObject **namespace_portion)
     return FL_MODULE_FOUND;
 }
 
+/*[clinic input]
+zipimport.zipimporter.find_module
+
+    fullname: unicode
+    path: object = None
+    /
+
+Search for a module specified by 'fullname'.
+
+'fullname' must be the fully qualified (dotted) module name. It returns the
+zipimporter instance itself if the module was found, or None if it wasn't.
+The optional 'path' argument is ignored -- it's there for compatibility
+with the importer protocol.
+
+[clinic start generated code]*/
 
-/* Check whether we can satisfy the import of the module named by
-   'fullname'. Return self if we can, None if we can't. */
 static PyObject *
-zipimporter_find_module(PyObject *obj, PyObject *args)
+zipimport_zipimporter_find_module_impl(ZipImporter *self, PyObject *fullname,
+                                       PyObject *path)
+/*[clinic end generated code: output=506087f609466dc7 input=e3528520e075063f]*/
 {
-    ZipImporter *self = (ZipImporter *)obj;
-    PyObject *path = NULL;
-    PyObject *fullname;
     PyObject *namespace_portion = NULL;
     PyObject *result = NULL;
 
-    if (!PyArg_ParseTuple(args, "U|O:zipimporter.find_module", &fullname, &path))
-        return NULL;
-
     switch (find_loader(self, fullname, &namespace_portion)) {
     case FL_ERROR:
         return NULL;
@@ -416,23 +460,31 @@ zipimporter_find_module(PyObject *obj, PyObject *args)
 }
 
 
-/* Check whether we can satisfy the import of the module named by
-   'fullname', or whether it could be a portion of a namespace
-   package. Return self if we can load it, a string containing the
-   full path if it's a possible namespace portion, None if we
-   can't load it. */
+/*[clinic input]
+zipimport.zipimporter.find_loader
+
+    fullname: unicode
+    path: object = None
+    /
+
+Search for a module specified by 'fullname'.
+
+'fullname' must be the fully qualified (dotted) module name. It returns the
+zipimporter instance itself if the module was found, a string containing the
+full path name if it's possibly a portion of a namespace package,
+or None otherwise. The optional 'path' argument is ignored -- it's
+there for compatibility with the importer protocol.
+
+[clinic start generated code]*/
+
 static PyObject *
-zipimporter_find_loader(PyObject *obj, PyObject *args)
+zipimport_zipimporter_find_loader_impl(ZipImporter *self, PyObject *fullname,
+                                       PyObject *path)
+/*[clinic end generated code: output=601599a43bc0f49a input=dc73f275b0d5be23]*/
 {
-    ZipImporter *self = (ZipImporter *)obj;
-    PyObject *path = NULL;
-    PyObject *fullname;
     PyObject *result = NULL;
     PyObject *namespace_portion = NULL;
 
-    if (!PyArg_ParseTuple(args, "U|O:zipimporter.find_module", &fullname, &path))
-        return NULL;
-
     switch (find_loader(self, fullname, &namespace_portion)) {
     case FL_ERROR:
         return NULL;
@@ -453,19 +505,27 @@ zipimporter_find_loader(PyObject *obj, PyObject *args)
     return result;
 }
 
-/* Load and return the module named by 'fullname'. */
+/*[clinic input]
+zipimport.zipimporter.load_module
+
+    fullname: unicode
+    /
+
+Load the module specified by 'fullname'.
+
+'fullname' must be the fully qualified (dotted) module name. It returns the
+imported module, or raises ZipImportError if it wasn't found.
+
+[clinic start generated code]*/
+
 static PyObject *
-zipimporter_load_module(PyObject *obj, PyObject *args)
+zipimport_zipimporter_load_module_impl(ZipImporter *self, PyObject *fullname)
+/*[clinic end generated code: output=7303cebf88d47953 input=c236e2e8621f04ef]*/
 {
-    ZipImporter *self = (ZipImporter *)obj;
     PyObject *code = NULL, *mod, *dict;
-    PyObject *fullname;
     PyObject *modpath = NULL;
     int ispackage;
 
-    if (!PyArg_ParseTuple(args, "U:zipimporter.load_module",
-                          &fullname))
-        return NULL;
     if (PyUnicode_READY(fullname) == -1)
         return NULL;
 
@@ -523,17 +583,22 @@ error:
     return NULL;
 }
 
-/* Return a string matching __file__ for the named module */
-static PyObject *
-zipimporter_get_filename(PyObject *obj, PyObject *args)
-{
-    ZipImporter *self = (ZipImporter *)obj;
-    PyObject *fullname, *code, *modpath;
-    int ispackage;
+/*[clinic input]
+zipimport.zipimporter.get_filename
 
-    if (!PyArg_ParseTuple(args, "U:zipimporter.get_filename",
-                          &fullname))
-        return NULL;
+    fullname: unicode
+    /
+
+Return the filename for the specified module.
+[clinic start generated code]*/
+
+static PyObject *
+zipimport_zipimporter_get_filename_impl(ZipImporter *self,
+                                        PyObject *fullname)
+/*[clinic end generated code: output=c5b92b58bea86506 input=28d2eb57e4f25c8a]*/
+{
+    PyObject *code, *modpath;
+    int ispackage;
 
     /* Deciding the filename requires working out where the code
        would come from if the module was actually loaded */
@@ -545,17 +610,23 @@ zipimporter_get_filename(PyObject *obj, PyObject *args)
     return modpath;
 }
 
-/* Return a bool signifying whether the module is a package or not. */
-static PyObject *
-zipimporter_is_package(PyObject *obj, PyObject *args)
-{
-    ZipImporter *self = (ZipImporter *)obj;
-    PyObject *fullname;
-    enum zi_module_info mi;
+/*[clinic input]
+zipimport.zipimporter.is_package
 
-    if (!PyArg_ParseTuple(args, "U:zipimporter.is_package",
-                          &fullname))
-        return NULL;
+    fullname: unicode
+    /
+
+Return True if the module specified by fullname is a package.
+
+Raise ZipImportError if the module couldn't be found.
+
+[clinic start generated code]*/
+
+static PyObject *
+zipimport_zipimporter_is_package_impl(ZipImporter *self, PyObject *fullname)
+/*[clinic end generated code: output=c32958c2a5216ae6 input=a7ba752f64345062]*/
+{
+    enum zi_module_info mi;
 
     mi = get_module_info(self, fullname);
     if (mi == MI_ERROR)
@@ -568,19 +639,35 @@ zipimporter_is_package(PyObject *obj, PyObject *args)
 }
 
 
+/*[clinic input]
+zipimport.zipimporter.get_data
+
+    pathname as path: unicode
+    /
+
+Return the data associated with 'pathname'.
+
+Raise OSError if the file was not found.
+
+[clinic start generated code]*/
+
 static PyObject *
-zipimporter_get_data(PyObject *obj, PyObject *args)
+zipimport_zipimporter_get_data_impl(ZipImporter *self, PyObject *path)
+/*[clinic end generated code: output=65dc506aaa268436 input=fa6428b74843c4ae]*/
 {
-    ZipImporter *self = (ZipImporter *)obj;
-    PyObject *path, *key;
+    PyObject *key;
     PyObject *toc_entry;
     Py_ssize_t path_start, path_len, len;
 
-    if (!PyArg_ParseTuple(args, "U:zipimporter.get_data", &path))
+    if (self->archive == NULL) {
+        PyErr_SetString(PyExc_ValueError,
+                        "zipimporter.__init__() wasn't called");
         return NULL;
+    }
 
 #ifdef ALTSEP
-    path = _PyObject_CallMethodId(path, &PyId_replace, "CC", ALTSEP, SEP);
+    path = _PyObject_CallMethodId((PyObject *)&PyUnicode_Type, &PyId_replace,
+                                  "OCC", path, ALTSEP, SEP);
     if (!path)
         return NULL;
 #else
@@ -603,7 +690,7 @@ zipimporter_get_data(PyObject *obj, PyObject *args)
         goto error;
     toc_entry = PyDict_GetItem(self->files, key);
     if (toc_entry == NULL) {
-        PyErr_SetFromErrnoWithFilenameObject(PyExc_IOError, key);
+        PyErr_SetFromErrnoWithFilenameObject(PyExc_OSError, key);
         Py_DECREF(key);
         goto error;
     }
@@ -615,28 +702,45 @@ zipimporter_get_data(PyObject *obj, PyObject *args)
     return NULL;
 }
 
+/*[clinic input]
+zipimport.zipimporter.get_code
+
+    fullname: unicode
+    /
+
+Return the code object for the specified module.
+
+Raise ZipImportError if the module couldn't be found.
+
+[clinic start generated code]*/
+
 static PyObject *
-zipimporter_get_code(PyObject *obj, PyObject *args)
+zipimport_zipimporter_get_code_impl(ZipImporter *self, PyObject *fullname)
+/*[clinic end generated code: output=b923c37fa99cbac4 input=2761412bc37f3549]*/
 {
-    ZipImporter *self = (ZipImporter *)obj;
-    PyObject *fullname;
-
-    if (!PyArg_ParseTuple(args, "U:zipimporter.get_code", &fullname))
-        return NULL;
-
     return get_module_code(self, fullname, NULL, NULL);
 }
 
-static PyObject *
-zipimporter_get_source(PyObject *obj, PyObject *args)
-{
-    ZipImporter *self = (ZipImporter *)obj;
-    PyObject *toc_entry;
-    PyObject *fullname, *subname, *path, *fullpath;
-    enum zi_module_info mi;
+/*[clinic input]
+zipimport.zipimporter.get_source
 
-    if (!PyArg_ParseTuple(args, "U:zipimporter.get_source", &fullname))
-        return NULL;
+    fullname: unicode
+    /
+
+Return the source code for the specified module.
+
+Raise ZipImportError if the module couldn't be found, return None if the
+archive does contain the module, but has no source for it.
+
+[clinic start generated code]*/
+
+static PyObject *
+zipimport_zipimporter_get_source_impl(ZipImporter *self, PyObject *fullname)
+/*[clinic end generated code: output=bc059301b0c33729 input=4e4b186f2e690716]*/
+{
+    PyObject *toc_entry;
+    PyObject *subname, *path, *fullpath;
+    enum zi_module_info mi;
 
     mi = get_module_info(self, fullname);
     if (mi == MI_ERROR)
@@ -677,84 +781,49 @@ zipimporter_get_source(PyObject *obj, PyObject *args)
     }
 
     /* we have the module, but no source */
-    Py_INCREF(Py_None);
-    return Py_None;
+    Py_RETURN_NONE;
 }
 
-PyDoc_STRVAR(doc_find_module,
-"find_module(fullname, path=None) -> self or None.\n\
-\n\
-Search for a module specified by 'fullname'. 'fullname' must be the\n\
-fully qualified (dotted) module name. It returns the zipimporter\n\
-instance itself if the module was found, or None if it wasn't.\n\
-The optional 'path' argument is ignored -- it's there for compatibility\n\
-with the importer protocol.");
+/*[clinic input]
+zipimport.zipimporter.get_resource_reader
 
-PyDoc_STRVAR(doc_find_loader,
-"find_loader(fullname, path=None) -> self, str or None.\n\
-\n\
-Search for a module specified by 'fullname'. 'fullname' must be the\n\
-fully qualified (dotted) module name. It returns the zipimporter\n\
-instance itself if the module was found, a string containing the\n\
-full path name if it's possibly a portion of a namespace package,\n\
-or None otherwise. The optional 'path' argument is ignored -- it's\n\
- there for compatibility with the importer protocol.");
+    fullname: unicode
+    /
 
-PyDoc_STRVAR(doc_load_module,
-"load_module(fullname) -> module.\n\
-\n\
-Load the module specified by 'fullname'. 'fullname' must be the\n\
-fully qualified (dotted) module name. It returns the imported\n\
-module, or raises ZipImportError if it wasn't found.");
+Return the ResourceReader for a package in a zip file.
 
-PyDoc_STRVAR(doc_get_data,
-"get_data(pathname) -> string with file data.\n\
-\n\
-Return the data associated with 'pathname'. Raise IOError if\n\
-the file wasn't found.");
+If 'fullname' is a package within the zip file, return the 'ResourceReader'
+object for the package.  Otherwise return None.
 
-PyDoc_STRVAR(doc_is_package,
-"is_package(fullname) -> bool.\n\
-\n\
-Return True if the module specified by fullname is a package.\n\
-Raise ZipImportError if the module couldn't be found.");
+[clinic start generated code]*/
 
-PyDoc_STRVAR(doc_get_code,
-"get_code(fullname) -> code object.\n\
-\n\
-Return the code object for the specified module. Raise ZipImportError\n\
-if the module couldn't be found.");
+static PyObject *
+zipimport_zipimporter_get_resource_reader_impl(ZipImporter *self,
+                                               PyObject *fullname)
+/*[clinic end generated code: output=5e367d431f830726 input=bfab94d736e99151]*/
+{
+    PyObject *module = PyImport_ImportModule("importlib.resources");
+    if (module == NULL) {
+        return NULL;
+    }
+    PyObject *retval = PyObject_CallMethod(
+        module, "_zipimport_get_resource_reader",
+        "OO", (PyObject *)self, fullname);
+    Py_DECREF(module);
+    return retval;
+}
 
-PyDoc_STRVAR(doc_get_source,
-"get_source(fullname) -> source string.\n\
-\n\
-Return the source code for the specified module. Raise ZipImportError\n\
-if the module couldn't be found, return None if the archive does\n\
-contain the module, but has no source for it.");
-
-
-PyDoc_STRVAR(doc_get_filename,
-"get_filename(fullname) -> filename string.\n\
-\n\
-Return the filename for the specified module.");
 
 static PyMethodDef zipimporter_methods[] = {
-    {"find_module", zipimporter_find_module, METH_VARARGS,
-     doc_find_module},
-    {"find_loader", zipimporter_find_loader, METH_VARARGS,
-     doc_find_loader},
-    {"load_module", zipimporter_load_module, METH_VARARGS,
-     doc_load_module},
-    {"get_data", zipimporter_get_data, METH_VARARGS,
-     doc_get_data},
-    {"get_code", zipimporter_get_code, METH_VARARGS,
-     doc_get_code},
-    {"get_source", zipimporter_get_source, METH_VARARGS,
-     doc_get_source},
-    {"get_filename", zipimporter_get_filename, METH_VARARGS,
-     doc_get_filename},
-    {"is_package", zipimporter_is_package, METH_VARARGS,
-     doc_is_package},
+    ZIPIMPORT_ZIPIMPORTER_FIND_MODULE_METHODDEF
+    ZIPIMPORT_ZIPIMPORTER_FIND_LOADER_METHODDEF
+    ZIPIMPORT_ZIPIMPORTER_LOAD_MODULE_METHODDEF
+    ZIPIMPORT_ZIPIMPORTER_GET_FILENAME_METHODDEF
+    ZIPIMPORT_ZIPIMPORTER_IS_PACKAGE_METHODDEF
+    ZIPIMPORT_ZIPIMPORTER_GET_DATA_METHODDEF
+    ZIPIMPORT_ZIPIMPORTER_GET_CODE_METHODDEF
+    ZIPIMPORT_ZIPIMPORTER_GET_SOURCE_METHODDEF
+    ZIPIMPORT_ZIPIMPORTER_GET_RESOURCE_READER_METHODDEF
     {NULL,              NULL}   /* sentinel */
 };
 
@@ -765,20 +834,6 @@ static PyMemberDef zipimporter_members[] = {
     {NULL}
 };
 
-PyDoc_STRVAR(zipimporter_doc,
-"zipimporter(archivepath) -> zipimporter object\n\
-\n\
-Create a new zipimporter instance. 'archivepath' must be a path to\n\
-a zipfile, or to a specific path inside a zipfile. For example, it can be\n\
-'/tmp/myimport.zip', or '/tmp/myimport.zip/mydirectory', if mydirectory is a\n\
-valid directory inside the archive.\n\
-\n\
-'ZipImportError is raised if 'archivepath' doesn't point to a valid Zip\n\
-archive.\n\
-\n\
-The 'archive' attribute of zipimporter objects contains the name of the\n\
-zipfile targeted.");
-
 #define DEFERRED_ADDRESS(ADDR) 0
 
 static PyTypeObject ZipImporter_Type = {
@@ -803,7 +858,7 @@ static PyTypeObject ZipImporter_Type = {
     0,                                          /* tp_as_buffer */
     Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE |
         Py_TPFLAGS_HAVE_GC,                     /* tp_flags */
-    zipimporter_doc,                            /* tp_doc */
+    zipimport_zipimporter___init____doc__,      /* tp_doc */
     zipimporter_traverse,                       /* tp_traverse */
     0,                                          /* tp_clear */
     0,                                          /* tp_richcompare */
@@ -818,7 +873,7 @@ static PyTypeObject ZipImporter_Type = {
     0,                                          /* tp_descr_get */
     0,                                          /* tp_descr_set */
     0,                                          /* tp_dictoffset */
-    (initproc)zipimporter_init,                 /* tp_init */
+    (initproc)zipimport_zipimporter___init__,   /* tp_init */
     PyType_GenericAlloc,                        /* tp_alloc */
     PyType_GenericNew,                          /* tp_new */
     PyObject_GC_Del,                            /* tp_free */
@@ -907,10 +962,8 @@ read_directory(PyObject *archive)
     fp = _Py_fopen_obj(archive, "rb");
     if (fp == NULL) {
         if (PyErr_ExceptionMatches(PyExc_OSError)) {
-            PyObject *exc, *val, *tb;
-            PyErr_Fetch(&exc, &val, &tb);
-            PyErr_Format(ZipImportError, "can't open Zip file: %R", archive);
-            _PyErr_ChainExceptions(exc, val, tb);
+            _PyErr_FormatFromCause(ZipImportError,
+                                   "can't open Zip file: %R", archive);
         }
         return NULL;
     }
@@ -1000,13 +1053,13 @@ read_directory(PyObject *archive)
             goto file_error;
         }
         name[name_size] = '\0';  /* Add terminating null byte */
-        if (SEP != '/') {
-            for (i = 0; i < name_size; i++) {
-                if (name[i] == '/') {
-                    name[i] = SEP;
-                }
+#if SEP != '/'
+        for (i = 0; i < name_size; i++) {
+            if (name[i] == '/') {
+                name[i] = SEP;
             }
         }
+#endif
         /* Skip the rest of the header.
          * On Windows, calling fseek to skip over the fields we don't use is
          * slower than reading the data because fseek flushes stdio's
@@ -1104,7 +1157,7 @@ get_decompress_func(void)
     _Py_IDENTIFIER(decompress);
 
     if (importing_zlib != 0)
-        /* Someone has a zlib.py[co] in their Zip file;
+        /* Someone has a zlib.pyc in their Zip file;
            let's avoid a stack overflow. */
         return NULL;
     importing_zlib = 1;
@@ -1196,7 +1249,7 @@ get_data(PyObject *archive, PyObject *toc_entry)
         goto file_error;
     }
     if (fread(buf, 1, data_size, fp) != (size_t)data_size) {
-        PyErr_SetString(PyExc_IOError,
+        PyErr_SetString(PyExc_OSError,
                         "zipimport: can't read data");
         goto error;
     }
@@ -1227,6 +1280,14 @@ get_data(PyObject *archive, PyObject *toc_entry)
     data = PyObject_CallFunction(decompress, "Oi", raw_data, -15);
     Py_DECREF(decompress);
     Py_DECREF(raw_data);
+    if (data != NULL && !PyBytes_Check(data)) {
+        PyErr_Format(PyExc_TypeError,
+                     "zlib.decompress() must return a bytes object, not "
+                     "%.200s",
+                     Py_TYPE(data)->tp_name);
+        Py_DECREF(data);
+        return NULL;
+    }
     return data;
 
 eof_error:
@@ -1263,7 +1324,7 @@ eq_mtime(time_t t1, time_t t2)
     return d <= 1;
 }
 
-/* Given the contents of a .py[co] file in a buffer, unmarshal the data
+/* Given the contents of a .pyc file in a buffer, unmarshal the data
    and return the code object. Return None if it the magic word doesn't
    match (we do this instead of raising an exception as we fall back
    to .py if available and we don't want to mask other errors).
@@ -1275,7 +1336,7 @@ unmarshal_code(PyObject *pathname, PyObject *data, time_t mtime)
     unsigned char *buf = (unsigned char *)PyBytes_AsString(data);
     Py_ssize_t size = PyBytes_Size(data);
 
-    if (size < 12) {
+    if (size < 16) {
         PyErr_SetString(ZipImportError,
                         "bad pyc data");
         return NULL;
@@ -1286,22 +1347,29 @@ unmarshal_code(PyObject *pathname, PyObject *data, time_t mtime)
             PySys_FormatStderr("# %R has bad magic\n",
                                pathname);
         }
-        Py_INCREF(Py_None);
-        return Py_None;  /* signal caller to try alternative */
+        Py_RETURN_NONE;  /* signal caller to try alternative */
     }
 
-    if (mtime != 0 && !eq_mtime(get_uint32(buf + 4), mtime)) {
+    uint32_t flags = get_uint32(buf + 4);
+    if (flags != 0) {
+        // Hash-based pyc. We currently refuse to handle checked hash-based
+        // pycs. We could validate hash-based pycs against the source, but it
+        // seems likely that most people putting hash-based pycs in a zipfile
+        // will use unchecked ones.
+        if (strcmp(_Py_CheckHashBasedPycsMode, "never") &&
+            (flags != 0x1 || !strcmp(_Py_CheckHashBasedPycsMode, "always")))
+            Py_RETURN_NONE;
+    } else if ((mtime != 0 && !eq_mtime(get_uint32(buf + 8), mtime))) {
         if (Py_VerboseFlag) {
             PySys_FormatStderr("# %R has bad mtime\n",
                                pathname);
         }
-        Py_INCREF(Py_None);
-        return Py_None;  /* signal caller to try alternative */
+        Py_RETURN_NONE;  /* signal caller to try alternative */
     }
 
     /* XXX the pyc's size field is ignored; timestamp collisions are probably
        unimportant with zip files. */
-    code = PyMarshal_ReadObjectFromString((char *)buf + 12, size - 12);
+    code = PyMarshal_ReadObjectFromString((char *)buf + 16, size - 16);
     if (code == NULL) {
         return NULL;
     }
@@ -1405,7 +1473,7 @@ get_mtime_of_source(ZipImporter *self, PyObject *path)
     PyObject *toc_entry, *stripped;
     time_t mtime;
 
-    /* strip 'c' or 'o' from *.py[co] */
+    /* strip 'c' from *.pyc */
     if (PyUnicode_READY(path) == -1)
         return (time_t)-1;
     stripped = PyUnicode_FromKindAndData(PyUnicode_KIND(path),
@@ -1460,6 +1528,12 @@ get_module_code(ZipImporter *self, PyObject *fullname,
     PyObject *path, *fullpath = NULL;
     struct st_zip_searchorder *zso;
 
+    if (self->prefix == NULL) {
+        PyErr_SetString(PyExc_ValueError,
+                        "zipimporter.__init__() wasn't called");
+        return NULL;
+    }
+
     subname = get_subname(fullname);
     if (subname == NULL)
         return NULL;
